from installclass import BaseInstallClass
from constants import *
from filer import *
from flags import flags
import os
import isys
import iutil
import packages
import shutil
import types
import yuminstall
try:
    import instnum
except ImportError:
    instnum = None

import gettext
_ = lambda x: gettext.ldgettext("anaconda", x)

import logging
log = logging.getLogger("anaconda")

# custom installs are easy :-)
class InstallClass(BaseInstallClass):
    # name has underscore used for mnemonics, strip if you dont need it
    id = "pld"
    name = N_("PLD Linux")
    pixmap = "custom.png"
    _description = N_("The default installation of %s includes a set of "
                     "software applicable for general internet usage. "
                     "What additional tasks would you like your system "
                     "to include support for?")
    _descriptionFields = (productName,)
    sortPriority = 10000
    allowExtraRepos = True

    repopaths = {
        "PLD Th - architecture specific": [
            "PLD/i686/RPMS",
        ],
        "PLD Th - generic packages": [
            "PLD/noarch/RPMS"
        ]
    }
    tasks = [
        (N_("Base"), [
            "basic",
            "base",
        ]),
        (N_("GNOME Desktop"), [
            "gnome",
            "gnome_complete",
            "gnome_games",
            "gnome_themes",
        ]),
        (N_("KDE Desktop"), [
            "kde_kdepim",
            "kde_kdeedu",
            "kde_multimedia",
            "kde_koffice",
            "kde_network",
            "kde_graphics",
            "kde_admin",
            "kde_games",
            "kde_look"
        ]),
        (N_("Basic IceWM"), [
            "icewm",
        ]),
        (N_("WindowMaker"), [
            "wmaker",
        ]),
        (N_("General Development Tools"), [
            "devel"
        ]),
        (N_("Java Development Tools"), [
            "java"
        ]),
    ]

    def setInstallData(self, anaconda):
        BaseInstallClass.setInstallData(self, anaconda)
        if not anaconda.isKickstart:
            BaseInstallClass.setDefaultPartitioning(self, 
                                                    anaconda.id.partitions,
                                                    CLEARPART_TYPE_LINUX)

    def setSteps(self, anaconda):
        dispatch = anaconda.dispatch
        BaseInstallClass.setSteps(self, anaconda);
        dispatch.skipStep("partition")
        dispatch.skipStep("regkey")

    def getPackagePaths(self, uri):
        rc = {}
        for (name, path) in self.repopaths.items():
            if not type(uri) == types.ListType:
                uri = [uri,]
            if not type(path) == types.ListType:
                path = [path,]

            lst = []
            for i in uri:
                for p in path:
                    lst.append("%s/%s" % (i, p))

            rc[name] = lst

        log.info("package paths is %s" %(rc,))
        return rc

    def postAction(self, anaconda):
        # FIXME: we only need to write fstab here because of rpm bug
        # see https://bugs.launchpad.net/pld-linux/+bug/262985
        upgrade = anaconda.id.getUpgrade()
        if not upgrade:
            anaconda.id.fsset.write(anaconda.rootPath)
        log.info("Going to bind /proc and /sys to run geninitrd")
        isys.mount("/proc", "%s/proc" %(anaconda.rootPath,), bindMount = 1)
        isys.mount("/sys", "%s/sys" %(anaconda.rootPath,), bindMount = 1)
        # find all installed kernels and try to generate initrd for each one
        kernels = packages.rpmKernelVersionList(anaconda.rootPath)
        for kernel in kernels:
            kernelVer = kernel[0]
            log.info("Running geninitrd for kernel %s" % kernelVer)
            iutil.execWithRedirect("/sbin/geninitrd", ["-v", "-f", "/boot/initrd-%s.gz" % kernelVer, kernelVer], root = anaconda.rootPath)
        log.info("Umounting /proc and /sys")
        isys.umount("%s/proc" %(anaconda.rootPath,))
        isys.umount("%s/sys" %(anaconda.rootPath,))
        # end workaround
        # try to get xorg into working state
        if os.path.exists("/etc/X11/xorg.conf") \
        and not os.path.exists("%s/etc/X11/xorg.conf" % anaconda.rootPath) \
        and os.path.exists("%s/etc/X11" % anaconda.rootPath):
            log.info("Copying xorg.conf to the new system")
            shutil.copyfile("/etc/X11/xorg.conf", "%s/etc/X11/xorg.conf" % anaconda.rootPath)

    def handleRegKey(self, key, intf, interactive = True):
        pass

    def getBackend(self):
        return yuminstall.YumBackend

    def __init__(self, expert):
        BaseInstallClass.__init__(self, expert)

